#!/bin/bash

if [ "$EUID" -ne 0 ]; then
  echo "This script must be run with superuser privileges. Please use 'sudo' to execute it."
  exit 1
fi

# Function to install whiptail on Ubuntu
install_whiptail_ubuntu() {
    if ! command -v whiptail &> /dev/null; then
        sudo apt-get update
        sudo apt-get install -y whiptail
    fi
}

# Function to install newt on CentOS/RHEL
install_newt_centos() {
    if ! command -v whiptail &> /dev/null; then
        sudo yum install -y newt
    fi
}

# Determine the package manager based on the system
if command -v apt-get &> /dev/null; then
    PACKAGE_MANAGER="apt"
    install_whiptail_ubuntu
elif command -v yum &> /dev/null; then
    PACKAGE_MANAGER="yum"
    install_newt_centos
else
    echo "Unsupported package manager. Aborting."
    exit 1
fi

whiptail --msgbox "Welcome to the CyLock SSH Configuration Wizard! Please Log onto Cylock Portal and get the Application ID and Application Key under Secure Application. " 10 50 --title "CyLock SSH Configuration"

AUTH_SERVER_URL=$(whiptail --inputbox "Enter the Auth Server URL:" 10 50 --title "CyLock SSH Configuration" 3>&1 1>&2 2>&3)
AUTHORIZATION=$(whiptail --inputbox "Enter the Application Key:" 10 50 --title "CyLock SSH Configuration" 3>&1 1>&2 2>&3)
ID_SP=$(whiptail --inputbox "Enter the Application ID:" 10 50 --title "CyLock SSH Configuration" 3>&1 1>&2 2>&3)
ENFORCE_DEFAULT_AUTH=$(whiptail --inputbox "Enforce Default Authentication (YES/NO)" 10 50 --title "CyLock SSH Configuration" 3>&1 1>&2 2>&3)
FAIL_MODE=$(whiptail --inputbox "Enter the Fail Mode (ALLOW/DENY):" 10 50 --title "CyLock SSH Configuration" 3>&1 1>&2 2>&3)
ALLOW_REGISTRATION=$(whiptail --inputbox "Do Want to Allow Device Registration (ALLOW/DENY):" 10 50 --title "CyLock SSH Configuration" 3>&1 1>&2 2>&3)

    
# Validate the input
if [ -z "$AUTH_SERVER_URL" ] || [ -z "$ID_SP" ] || [ -z "$AUTHORIZATION" ] || [ -z "$FAIL_MODE" ]; then
    echo "Error: One or more inputs are empty. Aborting."
    exit 1
fi

# Create the file /etc/cylock/ssh_conf.json and populate it with the user input
sudo mkdir -p /etc/cylock
sudo touch /etc/cylock/ssh_conf.json
sudo chmod 644 /etc/cylock/ssh_conf.json

# Create folder inside /var/log/cylock to capture log files.
sudo mkdir -p /var/log/cylock
#sudo chmod -R 766  /var/log/cylock/
sudo chmod -R 750 /var/log/cylock/


cat <<EOF | sudo tee /etc/cylock/ssh_conf.json
{
  "auth_server_url": "$AUTH_SERVER_URL",
  "id_sp": "$ID_SP",
  "authorization": "$AUTHORIZATION",
  "enforce_default_authentication": "$ENFORCE_DEFAULT_AUTH",
  "fail_mode": "$FAIL_MODE",
  "device_resgistration":"$ALLOW_REGISTRATION"
}
EOF

# Create the user cylock without a home directory
sudo useradd -r -s /bin/false cylock

# Create cylock group
sudo groupadd cylock

# Change the ownership of /etc/cylock/ssh_conf.json to cylock user
sudo chown -R cylock:cylock /etc/cylock/ssh_conf.json

# Add the user cylock to sudoers
sudo sh -c 'echo "cylock ALL=(ALL) NOPASSWD:ALL" > /etc/sudoers.d/cylock'

# Install the package using the appropriate package manager
if [ "$PACKAGE_MANAGER" == "apt" ]; then
    # Install DEB package using apt
    sudo dpkg -i CyLockSSH.deb
elif [ "$PACKAGE_MANAGER" == "yum" ]; then
    # Install RPM package using yum
    yum install -y CyLockSSH.rpm
fi

# Determine the package manager based on the system
if [ "$PACKAGE_MANAGER" == "apt" ]; then    
    sudo mkdir -p /lib/security
    sudo cp /etc/cylock/cylock_2fa_ssh.so /lib/security
    # Change ownership of the file cylock_2fa_ssh.so to cylock user
    sudo chown -R cylock:cylock /lib/security/cylock_2fa_ssh.so
    sudo chmod a+x  /lib/security/cylock_2fa_ssh.so

    echo "Updating CyLock PAM Configuration /etc/pam.d/sshd file"
    # Add the line after @include common-auth
    sudo sed -i '/@include common-auth/a auth required cylock_2fa_ssh.so' /etc/pam.d/sshd

    echo "Updating CyLock SSH Required Settings"
    sudo sed -i '/KbdInteractiveAuthentication no/d' /etc/ssh/sshd_config
    #printf "\nChallengeResponseAuthentication yes\nAuthenticationMethods publickey,password publickey,keyboard-interactive\n" | sudo tee -a /etc/ssh/sshd_config >/dev/null
    printf "\nKbdInteractiveAuthentication yes\nAuthenticationMethods password keyboard-interactive\n" | sudo tee -a /etc/ssh/sshd_config >/dev/null


    #Restart SSH
    sudo systemctl restart ssh

elif [ "$PACKAGE_MANAGER" == "yum" ]; then
    sudo mkdir -p /usr/lib64/security    
    # Change ownership of the file cylock_2fa_ssh.so to cylock user
    sudo chown -R cylock:cylock /usr/lib64/security/cylock_2fa_ssh.so
    sudo chmod a+x  /usr/lib64/security/cylock_2fa_ssh.so

    #SELinux Policy Creation and install 
    cd /usr/lib64/security/
    sudo audit2allow -M cylock_2fa_ssh.so -i /var/log/audit/audit.log
    semodule -i cylock_2fa_ssh.so.pp
    sudo semodule -l | grep cylock*

    rm -rf cylock_2fa_ssh.so.pp
    rm -rf cylock_2fa_ssh.so.te

    echo "Updating CyLock PAM Configuration /etc/pam.d/sshd file"
    # Add the lines to the existing section
    sudo sed -i '/-auth      optional     pam_reauthorize.so prepare/a auth    required      pam_unix.so     no_warn try_first_pass\nauth    required      cylock_2fa_ssh.so' /etc/pam.d/sshd

    echo "Updating CyLock SSH Required Settings"
    sudo sed -i '/ChallengeResponseAuthentication no/d' /etc/ssh/sshd_config
    #printf "\nChallengeResponseAuthentication yes\nAuthenticationMethods publickey,password publickey,keyboard-interactive\n" | sudo tee -a /etc/ssh/sshd_config >/dev/null
    printf "\nChallengeResponseAuthentication yes\nAuthenticationMethods password keyboard-interactive\n" | sudo tee -a /etc/ssh/sshd_config >/dev/null


    #Restart SSH
    sudo systemctl restart sshd

fi

echo "Installation and configuration completed successfully!"
